<?php
//
// dicom.php
//
// Module for client-side applications using DICOM 3.0 protocols
//
// CopyRight (c) 2003-2004 Xiaohui Li
//

// constant definitions
// abstract syntax definitions
$C_ECHO = '1.2.840.10008.1.1';
$C_FIND = '1.2.840.10008.5.1.4.1.2.1.1';
$C_MOVE = '1.2.840.10008.5.1.4.1.2.1.2';
// read/write buffer size
$MAX_PDU_LEN = 16 * 1024;

class BaseObject {

    function BaseObject() {
        $args = func_get_args();
        register_shutdown_function( array( &$this, '_destructor' ) );
        call_user_func_array( array( &$this, '_constructor' ), $args );
    }
    function _constructor() { }
    function _destructor() { }
}

class CNameType extends BaseObject {
	var $name;
	var $type;

    function _constructor($name, $type) {
		$this->name = $name;
		$this->type = $type;
	}
    function _destructor() { }
}

// attribute tag to (name, type) table
$ATTR_TBL = array(
	0x00080018 => new CNameType("SOP Instance UID", "a"),
	0x00080020 => new CNameType("Study Date", "A"),
	0x00080050 => new CNameType("Accession Number", "A"),
	0x00080052 => new CNameType("Query/Retrieve Level", "A"),
	0x00080060 => new CNameType("Modality", "A"),
	0x00080090 => new CNameType("Referring Physician's Name", "A"),
	0x00081030 => new CNameType("Study Description", "A"),
	0x00100010 => new CNameType("Patient' Name", "A"),
	0x00100020 => new CNameType("Patient ID", "A"),
	0x00100030 => new CNameType("Patient' Birth Date", "A"),
	0x00100040 => new CNameType("Patient' Sex", "A"),
	0x00180015 => new CNameType("Body Part Examined", "A"),
	0x0020000d => new CNameType("Study Instance UID", "a"),
	0x0020000e => new CNameType("Series Instance UID", "a"),
	0x00200010 => new CNameType("Study ID", "A"),
	0x00200013 => new CNameType("Instance Number", "A"),
	0x00201208 => new CNameType("Number of Study Related Instances", "A"),
	0x00201209 => new CNameType("Number of Series Related Instances", "A"),
	0x00280002 => new CNameType("Samples Per Pixel", "v"),
	0x00280010 => new CNameType("Rows", "v"),
	0x00280011 => new CNameType("Columns", "v"),
	0x00280100 => new CNameType("Bits Allocated", "v"),
	0x00280101 => new CNameType("Bits Stored", "v"),
);

// gobal variables
$sequence = 0;

// utility functions
function pacsone_dump(&$data) {
	$length = strlen($data);
    print "<table width=100% cellpadding=0 cellspacing=0 border=1>\n";
	for ($i = 0; $i < $length; $i++) {
		if ($i % 16 == 0)
			print "<tr>";
		print "<td>" . dechex(ord($data{$i})) . "</td>";
		if ($i % 16 == 15)
			print "</tr>";
	}
	if ($i % 16 != 15)
		print "</tr>";
	print "</table>";
}

class ApplicationContext extends BaseObject {
    var $data;
    // constant definitions
    var $CONTEXT = '1.2.840.10008.3.1.1.1';

    function _constructor() {
        $this->data = pack('a' . strlen($this->CONTEXT), $this->CONTEXT);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x10, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class AbstractSyntax extends BaseObject {
    var $uid;
    var $data;

    function _constructor($uid) {
        $this->uid = $uid;
        $this->data = pack('a' . strlen($this->uid), $this->uid);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x30, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class TransferSyntax extends BaseObject {
    var $data;
    // constant definitions
    var $SYNTAX = '1.2.840.10008.1.2';

    function _constructor() {
        $this->data = pack('a' . strlen($this->SYNTAX), $this->SYNTAX);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x40, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class PresentationContext extends BaseObject {
    var $ctxId;
    var $abstract;
    var $data;

    function _constructor($abs) {
        global $sequence;
        // presentation context ID
        $this->ctxId = ($sequence++ * 2 + 1) % 256;
        $this->data = pack('C4', $this->ctxId, 0, 0, 0);
        // abstract syntax
        $this->abstract = $abs;
        $absSyntax = new AbstractSyntax($this->abstract);
        $this->data .= $absSyntax->getDataBuffer();
        // transfer syntax
        $xferSyntax = new TransferSyntax();
        $this->data .= $xferSyntax->getDataBuffer();
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x20, 0, strlen($this->data));
        return ($header . $this->data);
    }
    function getContextId() {
        return $this->ctxId;
    }
}

class MaximumPduLength extends BaseObject {
    var $maxPduLen;
    var $data;

    function _constructor() {
        global $MAX_PDU_LEN;
        $this->maxPduLen = $MAX_PDU_LEN;
        $this->data = pack('N', $this->maxPduLen);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x51, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class ImpClassUid extends BaseObject {
    var $data;
	// constant definitions
	var $PACSONE_UID = '1.2.826.0.1.3680043.2.737';

    function _constructor() {
        $this->data = pack('a' . strlen($this->PACSONE_UID), $this->PACSONE_UID);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x52, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class UserInformation extends BaseObject {
    var $data;

    function _constructor() {
        $maxLen = new MaximumPduLength();
        $this->data = $maxLen->getDataBuffer();
		$uid = new ImpClassUid();
		$this->data .= $uid->getDataBuffer();
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCn', 0x50, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class AssociateRequestPdu extends BaseObject {
    var $abstract;
    var $calledAe;
    var $callingAe;
    var $lastCtxId;
    var $data;

    function _constructor($abs, $called, $calling) {
        $this->abstract = $abs;
        $this->calledAe = $called;
        $this->callingAe = $calling;
        // build content of ASSOCIATE-RQ pdu
        $this->data = pack('nn', 0x1, 0);
        // add called AE title
        $this->data .= pack('A16', $this->calledAe);
        // add calling AE title
        $this->data .= pack('A16', $this->callingAe);
        // add reserved 32-bytes
        $this->data .= pack('N8', 0, 0, 0, 0, 0, 0, 0, 0);
        // add Application Context
        $applContext = new ApplicationContext();
        $this->data .= $applContext->getDataBuffer();
        // add Presentation Context
        $presContext = new PresentationContext($this->abstract);
        $this->lastCtxId = $presContext->getContextId();
        $this->data .= $presContext->getDataBuffer();
        // add User Information
        $userInfo = new UserInformation();
        $this->data .= $userInfo->getDataBuffer();
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCN', 0x1, 0, strlen($this->data));
        return ($header . $this->data);
    }
    function getPresentContextId() {
        return $this->lastCtxId;
    }
    function isAccepted(&$data, &$error) {
        $result = false;
        // skip all the way to Application Context
        $skip = 68;
        $appl = substr($data, $skip);
		#pacsone_dump($appl);
        // skip the Application Context
        $header = substr($appl, 0, 4);
		$length = 4;
        $array = unpack('Ctype/Cdummy/nlength', $header);
        $length += $array['length'];
        $present = substr($appl, $length);
		#pacsone_dump($present);
        // check the Presentation Context item
        $itemType = ord($present{0});
        $ctxId = ord($present{4});
        $reason = ord($present{6});
        if ( ($itemType == 0x21) && ($ctxId == $this->lastCtxId) &&
             ($reason == 0) ) {
            $result = true;
        }
        else if ($itemType == 0x3) {
            $error = 'Association request rejected, reason = ' . $reason;
        } else {
            $error = 'Invalid Presentation Context received, type = ' . $itemType;
        }
        return $result;
    }
}

class CEchoPdv extends BaseObject {
    var $msgId;
    var $data;

    function _constructor() {
        global $C_ECHO;
        global $sequence;
        // write Affected SOP Class UID
        $length = strlen($C_ECHO);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2Va' . $length, 0, 2, $length, $C_ECHO);
        // write Command Field
        $this->data .= pack('v2Vv', 0, 0x100, 2, 0x30);
        // write Message ID
        $id = $sequence++ & 0xffffffff;
        $this->msgId = $id;
        $this->data .= pack('v2Vv', 0, 0x110, 2, $id);
        // write Data Set Type
        $this->data .= pack('v2Vv', 0, 0x800, 2, 0x101);
        #pacsone_dump($this->data);
    }
    function _destructor() { }
    function getDataBuffer() {
        // write Group Length
        $header = pack('v2V2', 0, 0, 4, strlen($this->data));
        return ($header . $this->data);
    }
    function recvResponse(&$data, &$complete, &$error) {
        $result = false;
        // skip to Group Length
        $data = substr($data, 6);
        // skip Group Length
        $data = substr($data, 12);
        // skip Affected SOP Class UID
        $header = substr($data, 0, 8);
        $array = unpack('vgroup/velement/Vlength', $header);
        $length = $array['length'];
        $data = substr($data, strlen($header) + $length);
        // check Command Field
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vcommand', $header);
        $command = $array['command'];
        if ($command != 0x8030) {
            $error = 'Invalid response received: command = ' . $command;
            return false;
        }
        $data = substr($data, strlen($header));
        // check Message ID
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vid', $header);
        $id = $array['id'];
        if ($id != $this->msgId) {
            $error = 'Message ID mismatch: received = ' . $id . ', expected = ' . $this->msgId;
            return false;
        }
        $data = substr($data, strlen($header));
        // skip Data Set type
        $data = substr($data, 10);
        // check Status
        $array = unpack('vgroup/velement/Vsize/vstatus', $data);
        $status = $array['status'];
        if ($status != 0) {
            $error = 'Command failed, response status = ' . $status;
        }
        else {
            $complete = true;
            $result = true;
        }
        return $result;
    }
}

class CMatchResult extends BaseObject {
	var $attrs = array();

    function _constructor($data) {
		global $ATTR_TBL;
        $total = strlen($data);
        while ($total > 0) {
            $header = substr($data, 0, 8);
            $array = unpack('vgroup/velement/Vlength', $header);
            $group = $array['group'];
            $element = $array['element'];
			$key = $group << 16 | $element;
            $length = $array['length'];
			$body = substr($data, strlen($header), $length);
			$format = $ATTR_TBL[$key]->type;
			if (strcasecmp($format{0}, "A") == 0)
				$format .= $length;
			$format .= 'value';
			$array = unpack($format, $body);
			$value = $array['value'];
			// save this attribute
			if ($format{0} == 'A')
				$value = str_replace("^", " ", $value);
			$this->attrs[$key] = $value;
            // next attribute
            $bytes = strlen($header) + $length;
            $data = substr($data, $bytes);
            $total -= $bytes;
        }
	}
    function _destructor() { }
	function hasKey($key) {
		return isset($this->attrs[$key]);
	}
	function getQueryLevel() {
		return $this->attrs[0x00080052];
	}
	function getPatientId() {
		return $this->attrs[0x00100020];
	}
	function getStudyUid() {
		return $this->attrs[0x0020000d];
	}
	function getSeriesUid() {
		return $this->attrs[0x0020000e];
	}
	function getSopInstanceUid() {
		return $this->attrs[0x00080018];
	}
}

class CFailedSopInstances extends BaseObject {
	var $list = array();

    function _constructor($data) {
        $header = substr($data, 0, 8);
        $array = unpack('vgroup/velement/Vlength', $header);
        $length = $array['length'];
		$body = substr($data, strlen($header), $length);
		$array = unpack('a' . $length . 'value', $body);
		$value = $array['value'];
		// split the '\' separated UIDs
		$this->list = explode("\\", $value);
	}
    function _destructor() { }
}

class CFindPdv extends BaseObject {
    var $finalStatus;
    var $msgId;
    var $data;
	// C-FIND response status table
	var $STATUS_TBL = array (
		0xA700 => "Refused: Out of Resources",
		0xA900 => "Failed: Identifier does not match SOP class",
		0xFE00 => "Cancel: Matching terminated due to Cancel request",
	);

    function _constructor() {
        global $C_FIND;
        global $sequence;
        // write Affected SOP Class UID
        $length = strlen($C_FIND);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2Va' . $length, 0, 2, $length, $C_FIND);
        // write Command Field
        $this->data .= pack('v2Vv', 0, 0x100, 2, 0x20);
        // write Message ID
        $id = $sequence++ & 0xffffffff;
        $this->msgId = $id;
        $this->data .= pack('v2Vv', 0, 0x110, 2, $id);
        // write Priority
        $this->data .= pack('v2Vv', 0, 0x700, 2, 0);
        // write Data Set Type
        $this->data .= pack('v2Vv', 0, 0x800, 2, 0);
        #pacsone_dump($this->data);
		$this->finalStatus = 0xFF00;
    }
    function _destructor() { }
    function getDataBuffer() {
        // write Group Length
        $header = pack('v2V2', 0, 0, 4, strlen($this->data));
        return ($header . $this->data);
    }
    function recvResponse(&$data, &$complete, &$error) {
		// check PDV header to see if it's a Command or Dataset
		$msgHdr = ord($data{5});
        // skip to Group Length
        $data = substr($data, 6);
		if ($msgHdr & 0x1) {
			$result = $this->recvCmdResponse($data, $error);
		} else {
			$result = $this->recvDataset($data, $error);
		}
		if ($this->finalStatus != 0xFF00) {
			if ($this->finalStatus)
				$error = "C-MOVE Response: " . $this->STATUS_TBL[ $this->finalStatus ];
			$complete = true;
		}
		return $result;
	}
    function recvCmdResponse(&$data, &$error) {
        $result = false;
        // skip Group Length
        $data = substr($data, 12);
        // skip Affected SOP Class UID
        $header = substr($data, 0, 8);
        $array = unpack('vgroup/velement/Vlength', $header);
        $length = $array['length'];
        $data = substr($data, strlen($header) + $length);
        // check Command Field
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vcommand', $header);
        $command = $array['command'];
        if ($command != 0x8020) {
            $error = 'Invalid response received: command = ' . $command;
            return false;
        }
        $data = substr($data, strlen($header));
        // check Message ID
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vid', $header);
        $id = $array['id'];
        if ($id != $this->msgId) {
            $error = 'Message ID mismatch: received = ' . $id . ', expected = ' . $this->msgId;
            return false;
        }
        $data = substr($data, strlen($header));
        // check Data Set type
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vtype', $header);
        $group = $array['group'];
        $element = $array['element'];
		if ($group != 0 or $element != 0x800) {
            $error = "Invalid Dataset Type: group = " . dechex($group);
			$error .= " element = " . dechex($element);
            return false;
		}
        $type = $array['type'];
        $data = substr($data, strlen($header));
        // check Status
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vstatus', $header);
        $status = $array['status'];
        if ($status != 0xFF00) {
			$this->finalStatus = $status;
            $result = false;
        }
        // parse returned Identifier
        if ($type != 0x101) {   // dataset present
            $data = substr($data, strlen($header));
        	// skip the Data Set PDV header
        	$data = substr($data, 6);
			$result = $this->recvDataset($data, $error);
        }
        return $result;
    }
    function recvDataset(&$data, &$error) {
		$result = new CMatchResult($data);
		return $result;
	}
}

class CMovePdv extends BaseObject {
    var $msgId;
    var $finalStatus;
    var $numRemaining;
    var $numCompleted;
    var $numFailed;
    var $numWarning;
    var $data;
	// C-MOVE response status table
	var $STATUS_TBL = array (
		0xA701 => "Refused: Out of Resources - Unable to calculate number of matches",
		0xA702 => "Refused: Out of Resources - Unable to perform sub-operations",
		0xA801 => "Refused: Move Destination Unknown",
		0xA900 => "Failed: Identifier does not match SOP class",
		0xFE00 => "Cancel: Sub-operations terminated due to Cancel indication",
		0xB000 => "Warning: Sub-operations Complete - One or more failures",
	);

    function _constructor($dest) {
        global $C_MOVE;
        global $sequence;
        // write Affected SOP Class UID
        $length = strlen($C_MOVE);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2Va' . $length, 0, 2, $length, $C_MOVE);
        // write Command Field
        $this->data .= pack('v2Vv', 0, 0x100, 2, 0x21);
        // write Message ID
        $id = $sequence++ & 0xffffffff;
        $this->msgId = $id;
        $this->data .= pack('v2Vv', 0, 0x110, 2, $id);
		// write Move Destination
		$length = strlen($dest);
		if ($length & 0x1)
			$length++;
		$this->data .= pack('v2VA' . $length, 0, 0x600, $length, $dest);
        // write Priority
        $this->data .= pack('v2Vv', 0, 0x700, 2, 0);
        // write Data Set Type
        $this->data .= pack('v2Vv', 0, 0x800, 2, 0);
        #pacsone_dump($this->data);
		$this->finalStatus = 0xFF00;
		$this->numRemaining = 0;
		$this->numCompleted = 0;
		$this->numFailed = 0;
		$this->numWarning = 0;
    }
    function _destructor() { }
    function getDataBuffer() {
        // write Group Length
        $header = pack('v2V2', 0, 0, 4, strlen($this->data));
        return ($header . $this->data);
    }
    function recvResponse(&$data, &$complete, &$error) {
		// check PDV header to see if it's a Command or Dataset
		$msgHdr = ord($data{5});
        // skip to Group Length
        $data = substr($data, 6);
		if ($msgHdr & 0x1) {
			$result = $this->recvCmdResponse($data, $error);
		} else {
			$result = $this->recvDataset($data, $error);
		}
		if ($this->finalStatus != 0xFF00) {
			if ($this->finalStatus) {
				$error = "C-MOVE Response: ";
				if (($this->finalStatus & 0xC000) == 0xC000)
					$error .= "Failed: Unable to Process";
				else
					$error .= $this->STATUS_TBL[ $this->finalStatus ];
			}
			$complete = true;
		}
		return $result;
	}
    function recvCmdResponse(&$data, &$error) {
        $result = true;
        // skip Group Length
        $data = substr($data, 12);
        // skip Affected SOP Class UID
        $header = substr($data, 0, 8);
        $array = unpack('vgroup/velement/Vlength', $header);
        $length = $array['length'];
        $data = substr($data, strlen($header) + $length);
        // check Command Field
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vcommand', $header);
        $command = $array['command'];
        if ($command != 0x8021) {
            $error = 'Invalid response received: command = ' . $command;
            return false;
        }
        $data = substr($data, strlen($header));
        // check Message ID
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vid', $header);
        $id = $array['id'];
        if ($id != $this->msgId) {
            $error = 'Message ID mismatch: received = ' . $id . ', expected = ' . $this->msgId;
            return false;
        }
        $data = substr($data, strlen($header));
        // check Data Set type
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vtype', $header);
        $type = $array['type'];
        $data = substr($data, strlen($header));
        // check Status
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vstatus', $header);
        $status = $array['status'];
        $data = substr($data, strlen($header));
		// check Number of Remaining Sub-operations
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vnumber', $header);
        $this->numRemaining = $array['number'];
        $data = substr($data, strlen($header));
		// check Number of Completed Sub-operations
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vnumber', $header);
        $this->numCompleted = $array['number'];
        $data = substr($data, strlen($header));
		// check Number of Failed Sub-operations
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vnumber', $header);
        $this->numFailed = $array['number'];
        $data = substr($data, strlen($header));
		// check Number of Warning Sub-operations
        $header = substr($data, 0, 10);
        $array = unpack('vgroup/velement/Vsize/vnumber', $header);
        $this->numWarning = $array['number'];
        $data = substr($data, strlen($header));
		// check final status
        if ($status != 0xFF00) {
			$this->finalStatus = $status;
            $result = false;
        }
        // parse returned Identifier
        if ($type != 0x101) {   // dataset present
        	// skip the Data Set PDV header
        	$data = substr($data, 6);
			$result = $this->recvDataset($data, $error);
        }
        return $result;
    }
    function recvDataset(&$data, &$error) {
		// the Dataset should contain a list of failed SOP instances
		$result = new CFailedSopInstances($data);
		return $result;
	}
}

class CFindIdentifierRoot extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor() {
        // add Query/Retrieve Level
        $level = 'PATIENT';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient Name
        $this->data .= pack('v2V', 0x10, 0x10, 0);
		$this->attrs[] = 0x00100010;
        // add Patient ID
        $this->data .= pack('v2V', 0x10, 0x20, 0);
		$this->attrs[] = 0x00100020;
        // add Patient Birth Date
        $this->data .= pack('v2V', 0x10, 0x30, 0);
		$this->attrs[] = 0x00100030;
        // add Patient Sex
        $this->data .= pack('v2V', 0x10, 0x40, 0);
		$this->attrs[] = 0x00100040;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CFindIdentifierPatient extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($id) {
		// add Study Date
        $this->data = pack('v2V', 0x8, 0x20, 0);
		$this->attrs[] = 0x00080020;
		// add Accession Number
        $this->data .= pack('v2V', 0x8, 0x50, 0);
		$this->attrs[] = 0x00080050;
        // add Query/Retrieve Level
        $level = 'STUDY';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
		// add Referring Physician's Name
        $this->data .= pack('v2V', 0x8, 0x90, 0);
		$this->attrs[] = 0x00080090;
		// add Study Description
        $this->data .= pack('v2V', 0x8, 0x1030, 0);
		$this->attrs[] = 0x00081030;
        // add Patient ID
		$length = strlen($id);
		if ($length & 0x1)
			$length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $id);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $this->data .= pack('v2V', 0x20, 0xd, 0);
		$this->attrs[] = 0x0020000d;
		// add Study ID
        $this->data .= pack('v2V', 0x20, 0x10, 0);
		$this->attrs[] = 0x00200010;
		// add Number of Study Related Instances
        $this->data .= pack('v2V', 0x20, 0x1208, 0);
		$this->attrs[] = 0x00201208;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CFindIdentifierStudy extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $uid) {
        // add Query/Retrieve Level
        $level = 'SERIES';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Modality
        $this->data .= pack('v2V', 0x8, 0x60, 0);
		$this->attrs[] = 0x00080060;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
        // add Body Part Examined
        $this->data .= pack('v2V', 0x18, 0x15, 0);
		$this->attrs[] = 0x00180015;
		// add Study Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x20, 0xd, $length, $uid);
		$this->attrs[] = 0x0020000d;
		// add Series Instance UID
        $this->data .= pack('v2V', 0x20, 0xe, 0);
		$this->attrs[] = 0x0020000e;
		// add Number of Series Related Instances
        $this->data .= pack('v2V', 0x20, 0x1209, 0);
		$this->attrs[] = 0x00201209;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CFindIdentifierSeries extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $studyuid, $uid) {
		// add SOP Instance UID
        $this->data = pack('v2V', 0x8, 0x18, 0);
		$this->attrs[] = 0x00080018;
        // add Query/Retrieve Level
        $level = 'IMAGE';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $length = strlen($studyuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xd, $length, $studyuid);
		$this->attrs[] = 0x0020000d;
		// add Series Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xe, $length, $uid);
		$this->attrs[] = 0x0020000e;
		// add Instance Number
        $this->data .= pack('v2V', 0x20, 0x13, 0);
		$this->attrs[] = 0x00200013;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CFindIdentifierImage extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $studyuid, $seriesuid, $uid) {
		// add SOP Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2Va' . $length, 0x8, 0x18, $length, $uid);
		$this->attrs[] = 0x00080018;
        // add Query/Retrieve Level
        $level = 'IMAGE';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $length = strlen($studyuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xd, $length, $studyuid);
		$this->attrs[] = 0x0020000d;
		// add Series Instance UID
        $length = strlen($seriesuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xe, $length, $seriesuid);
		$this->attrs[] = 0x0020000e;
		// add Instance Number
        $this->data .= pack('v2V', 0x20, 0x13, 0);
		$this->attrs[] = 0x00200013;
		// add Samples Per Pixel
        $this->data .= pack('v2V', 0x28, 0x2, 0);
		$this->attrs[] = 0x00280002;
		// add Rows
        $this->data .= pack('v2V', 0x28, 0x10, 0);
		$this->attrs[] = 0x00280010;
		// add Columns
        $this->data .= pack('v2V', 0x28, 0x11, 0);
		$this->attrs[] = 0x00280011;
		// add Bits Allocated
        $this->data .= pack('v2V', 0x28, 0x100, 0);
		$this->attrs[] = 0x00280100;
		// add Bits Stored
        $this->data .= pack('v2V', 0x28, 0x101, 0);
		$this->attrs[] = 0x00280101;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CMoveIdentifierPatient extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid) {
        // add Query/Retrieve Level
        $level = 'PATIENT';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CMoveIdentifierStudy extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $uid) {
        // add Query/Retrieve Level
        $level = 'STUDY';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xd, $length, $uid);
		$this->attrs[] = 0x0020000d;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CMoveIdentifierSeries extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $studyuid, $uid) {
        // add Query/Retrieve Level
        $level = 'SERIES';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $length = strlen($studyuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xd, $length, $studyuid);
		$this->attrs[] = 0x0020000d;
		// add Series Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xe, $length, $uid);
		$this->attrs[] = 0x0020000e;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class CMoveIdentifierImage extends BaseObject {
    var $attrs = array();
    var $data;

    function _constructor($patientid, $studyuid, $seriesuid, $uid) {
		// add SOP Instance UID
        $length = strlen($uid);
        if ($length & 0x1)
            $length++;
        $this->data = pack('v2Va' . $length, 0x8, 0x18, $length, $uid);
		$this->attrs[] = 0x00080018;
        // add Query/Retrieve Level
        $level = 'IMAGE';
        $length = strlen($level);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x8, 0x52, $length, $level);
		$this->attrs[] = 0x00080052;
        // add Patient ID
        $length = strlen($patientid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2VA' . $length, 0x10, 0x20, $length, $patientid);
		$this->attrs[] = 0x00100020;
		// add Study Instance UID
        $length = strlen($studyuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xd, $length, $studyuid);
		$this->attrs[] = 0x0020000d;
		// add Series Instance UID
        $length = strlen($seriesuid);
        if ($length & 0x1)
            $length++;
        $this->data .= pack('v2Va' . $length, 0x20, 0xe, $length, $seriesuid);
		$this->attrs[] = 0x0020000e;
    }
    function _destructor() { }
    function getDataBuffer() {
        return $this->data;
    }
	function getAttributeName($attr) {
		global $ATTR_TBL;
		return $ATTR_TBL[$attr]->name;
	}
}

class ProtocolDataTfPdu extends BaseObject {
    var $ctxId;
    var $pdv;
    var $data;

    function _constructor($ctxId) {
        $this->ctxId = $ctxId;
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCN', 0x4, 0, strlen($this->data));
        return ($header . $this->data);
    }
    function sendCommandEcho() {
        $this->pdv = new CEchoPdv();
        $data = $this->pdv->getDataBuffer();
        $this->data = pack('N', strlen($data)+2);
        $this->data .= pack('C2', $this->ctxId, 0x3);
        $this->data .= $data;
        #pacsone_dump($this->data);
    }
    function sendCommandFind() {
        $this->pdv = new CFindPdv();
        $data = $this->pdv->getDataBuffer();
        $this->data = pack('N', strlen($data)+2);
        $this->data .= pack('C2', $this->ctxId, 0x3);
        $this->data .= $data;
        #pacsone_dump($this->data);
    }
    function sendCommandMove($dest) {
        $this->pdv = new CMovePdv($dest);
        $data = $this->pdv->getDataBuffer();
        $this->data = pack('N', strlen($data)+2);
        $this->data .= pack('C2', $this->ctxId, 0x3);
        $this->data .= $data;
        #pacsone_dump($this->data);
    }
    function sendDataSet(&$dataset) {
        $data = $dataset->getDataBuffer();
        $this->data = pack('N', strlen($data)+2);
        $this->data .= pack('C2', $this->ctxId, 0x2);
        $this->data .= $data;
    }
    function recvResponse(&$data, &$complete, &$error) {
        return $this->pdv->recvResponse($data, $complete, $error);
    }
}

class AssociateReleasePdu extends BaseObject {
    var $data;

    function _constructor() {
        $this->data = pack('N', 0);
    }
    function _destructor() { }
    function getDataBuffer() {
        $header = pack('CCN', 0x5, 0, strlen($this->data));
        return ($header . $this->data);
    }
}

class Association extends BaseObject {
    var $socket;
    var $ipAddr;
    var $hostName;
    var $tcpPort;
    var $calledAe;
    var $callingAe;
    var $accepted;
	var $connected = false;
    // constant definitions
    var $TIMEOUT = 5;
    var $MY_AE_TITLE = "PacsOne_Client";

    function _constructor($ip, $host, $port, $called) {
        $address = (strlen($ip))? $ip : $host;
        $errno = 0;
        $errstr = '';
        $this->socket = fsockopen($address, $port, $errno, $errstr, $this->TIMEOUT);
        if (!$this->socket)
            die ('<font color=red>fsockopen() failed: ' . $errstr . '(' . $errno . ')</font>');
        stream_set_timeout($this->socket, $this->TIMEOUT);
        $this->ipAddr = $ip;
        $this->hostName = $host;
        $this->tcpPort = $port;
        $this->calledAe = $called;
        $this->callingAe = $this->MY_AE_TITLE;
		$this->connected = true;
		$this->accepted = false;
    }
    function _destructor() {
		if ($this->connected) {
			// close the association
        	fclose($this->socket);
		}
    }
	// establish association
	function associate($sopClass, &$error) {
	    $request = new AssociateRequestPdu($sopClass, $this->calledAe, $this->callingAe);
        $data = $request->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // check if request is accepted
        $data = fread($this->socket, 6);
		#pacsone_dump($data);
        // check the PDU type
        if ($data && ord($data{0}) == 0x2) {
            $array = unpack('Ctype/Cdummy/Nlength', $data);
			$pduLen = $array['length'];
        	$data = fread($this->socket, $pduLen);
			while (strlen($data) < $pduLen) {
        		$data .= fread($this->socket, $pduLen);
			}
			#pacsone_dump($data);
           	$this->accepted = $request->isAccepted($data, $error);
        }
        else {
            $error = 'Association request rejected';
            return false;
        }
    	return $request;
	}
	// release association
	function release(&$error) {
		if ($this->accepted) {
			// send ASSOCIATE_RELEASE_RQ PDU
	       	$request = new AssociateReleasePdu();
       		$data = $request->getDataBuffer();
		    fwrite($this->socket, $data, strlen($data));
			// check received ASSOCIATE_RELEASE_RP PDU
   		    $data = fread($this->socket, 10);
			$pduType = ord($data{0});
            /*
   	     	if ($pduType != 0x6)
           		$error .= "Invalid ASSOCIATE_RELEASE_RSP PDU received: pduType = " . $pduType;
            */
		}
	}
    // send C-ECHO request
    function verify(&$error) {
        $result = false;
        global $C_ECHO;
		$request = $this->associate($C_ECHO, $error);
		if (!$request)
			return false;
        // send the C-ECHO command PDV
        $dataPdu = new ProtocolDataTfPdu( $request->getPresentContextId() );
        $dataPdu->sendCommandEcho();
        $data = $dataPdu->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // read the response for the C-ECHO command
        $data = fread($this->socket, 6);
        // check the PDU type
		$respComplete = false;
		do {
        	if ($data && ord($data{0}) == 0x4) {
            	$array = unpack('Ctype/Cdummy/Nlength', $data);
		    	$pduLen = $array['length'];
            	$data = fread($this->socket, $pduLen);
		    	while (strlen($data) < $pduLen) {
            		$data .= fread($this->socket, $pduLen);
		    	}
       	    	$result = $dataPdu->recvResponse($data, $respComplete, $error);
        	}
        } while (!$respComplete && !strlen($error));
		// release the association
		$this->release($error);
        return $result;
    }
    // send C-FIND request
    function find(&$identifier, &$error) {
        $result = array();
        global $C_FIND;
		$request = $this->associate($C_FIND, $error);
        if (!$request)
			return false;
        // send the C-FIND command PDV
        $dataPdu = new ProtocolDataTfPdu( $request->getPresentContextId() );
        $dataPdu->sendCommandFind();
        $data = $dataPdu->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // send the Identifier in a Data Set PDV 
        $dataPdu->sendDataSet($identifier);
        $data = $dataPdu->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // read the response for the C-FIND command
		$respComplete = false;
        do {
            $data = fread($this->socket, 6);
            // check the PDU type
            if ($data && ord($data{0}) == 0x4) {
                $array = unpack('Ctype/Cdummy/Nlength', $data);
		        $pduLen = $array['length'];
                $data = fread($this->socket, $pduLen);
		        while (strlen($data) < $pduLen) {
                	$data .= fread($this->socket, $pduLen);
		        }
       	        $match = $dataPdu->recvResponse($data, $respComplete, $error);
				if (!$match)
					break;
				$result[] = $match;
            }
        } while (!$respComplete && !strlen($error));
		// release the association
		$this->release($error);
        return $result;
    }
    // send C-MOVE request
    function move($dest, &$identifier, &$error) {
        $result = array();
        global $C_MOVE;
		$request = $this->associate($C_MOVE, $error);
        if (!$request)
			return false;
        // send the C-MOVE command PDV
        $dataPdu = new ProtocolDataTfPdu( $request->getPresentContextId() );
        $dataPdu->sendCommandMove($dest);
        $data = $dataPdu->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // send the Identifier in a Data Set PDV 
        $dataPdu->sendDataSet($identifier);
        $data = $dataPdu->getDataBuffer();
        fwrite($this->socket, $data, strlen($data));
        // read the response for the C-MOVE command
		$respComplete = false;
        do {
            $data = fread($this->socket, 6);
            // check the PDU type
            if ($data && ord($data{0}) == 0x4) {
                $array = unpack('Ctype/Cdummy/Nlength', $data);
		        $pduLen = $array['length'];
                $data = fread($this->socket, $pduLen);
		        while (strlen($data) < $pduLen) {
                	$data .= fread($this->socket, $pduLen);
		        }
       	        $match = $dataPdu->recvResponse($data, $respComplete, $error);
				if (!$match)
					break;
				if (is_a($match, 'CFailedSopInstances'))
					$result[] = $match;
            }
        } while (!$respComplete && !strlen($error));
		// release the association
		$this->release($error);
        return $result;
    }
}

?>
