<?php 

/* 

Zip file creation class 
makes zip files on the fly... 

use the functions add_dir() and add_file() to build the zip file; 
see example code below 

by Eric Mueller 
http://www.themepark.com 

v1.1 9-20-01 
  - added comments to example 

v1.0 2-5-01 

initial version with: 
  - class appearance 
  - add_file() and file() methods 
  - gzcompress() output hacking 
by Denis O.Philippov, webmaster@atlant.ru, http://www.atlant.ru 

*/   

// official ZIP file format: http://www.pkware.com/appnote.txt 

class zipfile    
{    

    var $datasec = "";
    var $ctrl_dir = array(); // central directory     
    var $eof_ctrl_dir = "\x50\x4b\x05\x06\x00\x00\x00\x00"; //end of Central directory record 
    var $old_offset = 0;   

    function add_dir($name)     

    // adds "directory" to archive - do this before putting any files in directory! 
    // $name - name of directory... like this: "path/" 
    // ...then you can add files using add_file with names like "path/file.txt" 
    {    
        $name = str_replace("\\", "/", $name);    

        $this->datasec .= "\x50\x4b\x03\x04";   
        $this->datasec .= "\x0a\x00";    // ver needed to extract 
        $this->datasec .= "\x00\x00";    // gen purpose bit flag 
        $this->datasec .= "\x00\x00";    // compression method 
        $this->datasec .= "\x00\x00\x00\x00"; // last mod time and date 

        $this->datasec .= pack("V",0); // crc32 
        $this->datasec .= pack("V",0); //compressed filesize 
        $this->datasec .= pack("V",0); //uncompressed filesize 
        $this->datasec .= pack("v", strlen($name) ); //length of pathname 
        $this->datasec .= pack("v", 0 ); //extra field length 
        $this->datasec .= $name;    
        // end of "local file header" segment 

        // no "file data" segment for path 

        // "data descriptor" segment (optional but necessary if archive is not served as file) 
        $this->datasec .= pack("V",$crc); //crc32 
        $this->datasec .= pack("V",$c_len); //compressed filesize 
        $this->datasec .= pack("V",$unc_len); //uncompressed filesize 

        $new_offset = strlen($this->datasec);   

        // ext. file attributes mirrors MS-DOS directory attr byte, detailed 
        // at http://support.microsoft.com/support/kb/articles/Q125/0/19.asp 

        // now add to central record 
        $cdrec = "\x50\x4b\x01\x02";   
        $cdrec .="\x00\x00";    // version made by 
        $cdrec .="\x0a\x00";    // version needed to extract 
        $cdrec .="\x00\x00";    // gen purpose bit flag 
        $cdrec .="\x00\x00";    // compression method 
        $cdrec .="\x00\x00\x00\x00"; // last mod time & date 
        $cdrec .= pack("V",0); // crc32 
        $cdrec .= pack("V",0); //compressed filesize 
        $cdrec .= pack("V",0); //uncompressed filesize 
        $cdrec .= pack("v", strlen($name) ); //length of filename 
        $cdrec .= pack("v", 0 ); //extra field length     
        $cdrec .= pack("v", 0 ); //file comment length 
        $cdrec .= pack("v", 0 ); //disk number start 
        $cdrec .= pack("v", 0 ); //internal file attributes 
        $ext = "\x00\x00\x10\x00";   
        $ext = "\xff\xff\xff\xff";    
        $cdrec .= pack("V", 16 ); //external file attributes  - 'directory' bit set 

        $cdrec .= pack("V", $this -> old_offset ); //relative offset of local header 
        $this -> old_offset = $new_offset;   

        $cdrec .= $name;    
        // optional extra field, file comment goes here 
        // save to array 
        $this -> ctrl_dir[] = $cdrec;    

           
    }   


    function add_file(&$data, $name)     

    // adds "file" to archive     
    // $data - file contents 
    // $name - name of file in archive. Add path if your want 

    {    
        $name = str_replace("\\", "/", $name);    
        //$name = str_replace("\\", "\\\\", $name); 

        $this->datasec .= "\x50\x4b\x03\x04";   
        $this->datasec .= "\x14\x00";    // ver needed to extract 
        $this->datasec .= "\x00\x00";    // gen purpose bit flag 
        $this->datasec .= "\x08\x00";    // compression method 
        $this->datasec .= "\x00\x00\x00\x00"; // last mod time and date 

        $unc_len = strlen($data);    
        $crc = crc32($data);    
        $zdata = gzcompress($data);    
        $zdata = substr( substr($zdata, 0, strlen($zdata) - 4), 2); // fix crc bug 
        $c_len = strlen($zdata);    
        $this->datasec .= pack("V",$crc); // crc32 
        $this->datasec .= pack("V",$c_len); //compressed filesize 
        $this->datasec .= pack("V",$unc_len); //uncompressed filesize 
        $this->datasec .= pack("v", strlen($name) ); //length of filename 
        $this->datasec .= pack("v", 0 ); //extra field length 
        $this->datasec .= $name;    
        // end of "local file header" segment 
           
        // "file data" segment 
        $this->datasec .= $zdata;    

        // "data descriptor" segment (optional but necessary if archive is not served as file) 
        $this->datasec .= pack("V",$crc); //crc32 
        $this->datasec .= pack("V",$c_len); //compressed filesize 
        $this->datasec .= pack("V",$unc_len); //uncompressed filesize 

        $new_offset = strlen($this->datasec);   

        // now add to central directory record 
        $cdrec = "\x50\x4b\x01\x02";   
        $cdrec .="\x00\x00";    // version made by 
        $cdrec .="\x14\x00";    // version needed to extract 
        $cdrec .="\x00\x00";    // gen purpose bit flag 
        $cdrec .="\x08\x00";    // compression method 
        $cdrec .="\x00\x00\x00\x00"; // last mod time & date 
        $cdrec .= pack("V",$crc); // crc32 
        $cdrec .= pack("V",$c_len); //compressed filesize 
        $cdrec .= pack("V",$unc_len); //uncompressed filesize 
        $cdrec .= pack("v", strlen($name) ); //length of filename 
        $cdrec .= pack("v", 0 ); //extra field length     
        $cdrec .= pack("v", 0 ); //file comment length 
        $cdrec .= pack("v", 0 ); //disk number start 
        $cdrec .= pack("v", 0 ); //internal file attributes 
        $cdrec .= pack("V", 32 ); //external file attributes - 'archive' bit set 

        $cdrec .= pack("V", $this -> old_offset ); //relative offset of local header 
//        echo "old offset is ".$this->old_offset.", new offset is $new_offset<br>"; 
        $this -> old_offset = $new_offset;   

        $cdrec .= $name;    
        // optional extra field, file comment goes here 
        // save to central directory 
        $this -> ctrl_dir[] = $cdrec;    
    }   

    function file() { // dump out file     
        $ctrldir = implode("", $this -> ctrl_dir);    

        return     
            $this->datasec.    
            $ctrldir.    
            $this -> eof_ctrl_dir.    
            pack("v", sizeof($this -> ctrl_dir)).	// total # of entries "on this disk" 
            pack("v", sizeof($this -> ctrl_dir)).   // total # of entries overall 
            pack("V", strlen($ctrldir)).            // size of central dir 
            pack("V", strlen($this->datasec)).      // offset to start of central dir 
            "\x00\x00";                             // .zip file comment length 
    }   
}    

function addFile(&$zipfile, $name, $path) {
	$handle = fopen($path, "rb");
	$data = fread($handle, filesize($path));
	fclose($handle);
    // add the binary data stored in the string 'filedata' 
    $zipfile -> add_file($data, "$name");    
	return $data;
}

session_start();

include 'database.php';
include 'security.php';

$uid = $_REQUEST['id'];
$level = $_REQUEST['level'];
$uid = urlClean($uid, 64);
$database = $_SESSION['authenticatedDatabase'];
$username = $_SESSION['authenticatedUser'];
$password = $_SESSION['authenticatedPassword'];
if (!db_open($hostname, $database, $username, $password))
    show_error();
$zipfile = new zipfile();
$filename = $level . "-" . $uid . ".zip";
if (strcasecmp($level, "Patient") == 0) {
	$studies = mysql_query("SELECT uid FROM study WHERE patientid='$uid';");
	while ($study = mysql_fetch_row($studies)) {
		$studyuid = $study[0];
		$series = mysql_query("SELECT uid FROM series WHERE studyuid='$studyuid';");
		while ($serie = mysql_fetch_row($series)) {
			$seriesuid = $serie[0];
			$images = mysql_query("SELECT uid,path FROM image WHERE seriesuid='$seriesuid';");
			while ($image = mysql_fetch_row($images)) {
				$uid = $image[0];
				$path = $image[1];
				addFile($zipfile, $uid, $path);
			}
		}
	}
} else if (strcasecmp($level, "Study") == 0) 	{
	$series = mysql_query("SELECT uid FROM series WHERE studyuid='$uid';");
	while ($serie = mysql_fetch_row($series)) {
		$seriesuid = $serie[0];
		$images = mysql_query("SELECT uid,path FROM image WHERE seriesuid='$seriesuid';");
		while ($image = mysql_fetch_row($images)) {
			$uid = $image[0];
			$path = $image[1];
			addFile($zipfile, $uid, $path);
		}
	}
} else if (strcasecmp($level, "Series") == 0) {
	$result = mysql_query("SELECT uid,path FROM image WHERE seriesuid='$uid';");
	while ($row = mysql_fetch_row($result)) {
		$uid = $row[0];
		$path = $row[1];
		addFile($zipfile, $uid, $path);
	}
} else if (strcasecmp($level, "Image") == 0) {
	$result = mysql_query("SELECT path FROM image WHERE uid='$uid';");
	$row = mysql_fetch_row($result);
	$path = $row[0];
	addFile($zipfile, $uid, $path);
} else {
	die ("<p><font color=red>Invalid LEVEL: $level.</font></p>");
}
$data = $zipfile -> file();
// MSIE handling of Content-Disposition
if (strstr($_SERVER["HTTP_USER_AGENT"], "MSIE"))
    $filename = str_replace(".", "%2E", $filename) . "%20";
// the next three lines force an immediate download of the zip file: 
header("Content-type: application/octet-stream");    
header("Content-disposition: attachment; filename=$filename");    
echo $data;    

?> 

